using System;
using System.Collections.Generic;
using System.Text;
using HikariDemo.Core;
using Makarui;
using Mogre;

namespace HikariDemo
{
    class HikariScene : CoreClass
    {
        MakaruiManager hikariMgr;
        FlashControl fps;
        FlashControl controls;
        CallbackDelegate AsyncExitCallback;
        CallbackDelegate AsyncOpacityCallback;
        CallbackDelegate AsyncColorCallback;
        CallbackDelegate AsyncTestCallback;

        public override void CreateFrameListener()
        {
            base.CreateFrameListener();
            root.FrameStarted += new FrameListener.FrameStartedHandler(Main_FrameStarted);
        }

        bool Main_FrameStarted(FrameEvent evt)
        {
            hikariMgr.Update();
            HandleInput(evt);

            

            RenderTarget.FrameStats stats = window.GetStatistics();

            object[] args = new object[1];
            args[0] = (int)stats.LastFPS;

            fps.CallFunction("setFPS", args);

            return !ShouldQuit;
        }

        public override void CreateScene()
        {
            AsyncExitCallback = this.onExitClick;
            AsyncOpacityCallback = this.onOpacityChange;
            AsyncColorCallback = this.onColorChange;
            AsyncTestCallback = this.test;


            hikariMgr = new MakaruiManager("..\\media\\",this.window);
            controls = hikariMgr.CreateFlashOverlay("controls.swf", this.viewport, 350, 400, 1, 0, true, ScaleMode.SHOWALL, RenderQuality.AUTOHIGH);
            controls.ManualInvalidation = false;

            viewport.BackgroundColour = new ColourValue(200, 200, 200);

            fps = hikariMgr.CreateFlashOverlay("fps.swf", viewport, 130, 91, 0, 0,true);
            fps.ManualInvalidation = false;
           
            controls.Bind("opacityChange", AsyncOpacityCallback);
            controls.Bind("colorChange", AsyncColorCallback);
            controls.Bind("exitClick", AsyncExitCallback);
        }

        public void test(object[] args)
        {
            Console.Out.WriteLine("test callback");
        }


        public override void DestroyScene()
        {
            hikariMgr.Destroy();
            hikariMgr = null;
        }

        public override void CreateInput()
        {
            base.CreateInput();

            inputMouse.MouseMoved += new MOIS.MouseListener.MouseMovedHandler(inputMouse_MouseMoved);
            inputMouse.MousePressed += new MOIS.MouseListener.MousePressedHandler(inputMouse_MousePressed);
            inputMouse.MouseReleased += new MOIS.MouseListener.MouseReleasedHandler(inputMouse_MouseReleased);
        }

        bool inputMouse_MouseReleased(MOIS.MouseEvent arg, MOIS.MouseButtonID id)
        {
            //return true;
            return hikariMgr.InjectMouseUp(arg,id);
            //throw new Exception("The method or operation is not implemented.");
        }

        bool inputMouse_MousePressed(MOIS.MouseEvent arg, MOIS.MouseButtonID id)
        {

                return hikariMgr.InjectMouseDown(arg, id);
            //throw new Exception("The method or operation is not implemented.");
        }

        bool inputMouse_MouseMoved(MOIS.MouseEvent arg)
        {
           // return true;
            return hikariMgr.InjectMouseMove(arg.state.X.abs, arg.state.Y.abs);
            
            //throw new Exception("The method or operation is not implemented.");
        }

        void onExitClick(object[] args)
        {
            ShouldQuit = true;
        }

        void onOpacityChange(object[] args)
        {
           float Value = Convert.ToSingle(args[0]);
         //  viewport.BackgroundColour = new ColourValue(red, 200, 200);
           controls.SetOpacity(Value / 100);
        }

        void onColorChange(object[] args)
        {
            viewport.BackgroundColour = Utilities.GetNumberAsColor(Convert.ToSingle(args[0]));
        }
    }
}
